@@ -32,7 +32,7 @@ getRangeBounds <- function(pp) {
 #'
 #' The Box-Cox (\code{method = "BoxCox"}), Yeo-Johnson (\code{method =
 #' "YeoJohnson"}), and exponential transformations (\code{method =
-#' "expoTrans"})have been "repurposed" here: they are being used to transform
+#' "expoTrans"}) have been "repurposed" here: they are being used to transform
 #' the predictor variables. The Box-Cox transformation was developed for
 #' transforming the response variable while another method, the Box-Tidwell
 #' transformation, was created to estimate transformations of predictor data.
@@ -40,7 +40,7 @@ getRangeBounds <- function(pp) {
 #' is equally effective for estimating power transformations. The Yeo-Johnson
 #' transformation is similar to the Box-Cox model but can accommodate
 #' predictors with zero and/or negative values (while the predictors values for
-#' the Box-Cox transformation must be strictly positive.) The exponential
+#' the Box-Cox transformation must be strictly positive). The exponential
 #' transformation of Manly (1976) can also be used for positive or negative
 #' data.
 #'
@@ -52,7 +52,8 @@ getRangeBounds <- function(pp) {
 #' samples have values larger or smaller than those in the training set, values
 #' will be outside of this range.
 #'
-#' Predictors that are not numeric are ignored in the calculations.
+#' Predictors that are not numeric are ignored in the calculations (including
+#' methods "zv`" and "nzv`").
 #'
 #' \code{method = "zv"} identifies numeric predictor columns with a single
 #' value (i.e. having zero variance) and excludes them from further
@@ -98,7 +99,7 @@ getRangeBounds <- function(pp) {
 #' less than two unique values unless either \code{method = "zv"} or
 #' \code{method = "nzv"} are invoked.
 #'
-#' Non-numeric data will not be pre-processed and there values will be in the
+#' Non-numeric data will not be pre-processed and their values will be in the
 #' data frame produced by the \code{predict} function. Note that when PCA or
 #' ICA is used, the non-numeric columns may be in different positions when
 #' predicted.
@@ -165,7 +166,7 @@ getRangeBounds <- function(pp) {
 #' (chapter 4)
 #'
 #' Kuhn (2008), Building predictive models in R using the caret
-#' (\url{http://www.jstatsoft.org/article/view/v028i05/v28i05.pdf})
+#' (\url{https://www.jstatsoft.org/article/view/v028i05/v28i05.pdf})
 #'
 #' Box, G. E. P. and Cox, D. R. (1964) An analysis of transformations (with
 #' discussion). Journal of the Royal Statistical Society B, 26, 211-252.
@@ -236,9 +237,11 @@ preProcess.default <- function(x, method = c("center", "scale"),
   ## check for zero-variance predictors
   if(any(names(method) == "zv")){
     if(is.data.frame(x)) {
-      is_zv <- unlist(lapply(x[, !(colnames(x) %in% method$ignore), drop = FALSE], function(x) length(unique(x)) <= 1))
+      is_zv <- unlist(lapply(x[, !(colnames(x) %in% method$ignore), drop = FALSE], function(x)
+        ifelse(na.remove, length(na.omit(unique(x))), length(unique(x))) <= 1))
     } else {
-      is_zv <- apply(x[, !(colnames(x) %in% method$ignore), drop = FALSE], 2, function(x) length(unique(x)) <= 1)
+      is_zv <- apply(x[, !(colnames(x) %in% method$ignore), drop = FALSE], 2, function(x)
+        ifelse(na.remove, length(na.omit(unique(x))), length(unique(x))) <= 1)
     }
     if(any(is_zv)) {
       removed <- names(is_zv)[is_zv]
@@ -276,17 +279,17 @@ preProcess.default <- function(x, method = c("center", "scale"),
 
   ## check for highly correlated predictors
   if(any(names(method) == "corr")){
-    cmat <- try(cor(x[, !(colnames(x) %in% c(method$ignore, method$remove)), drop = FALSE], 
-                    use = "pairwise.complete.obs"), 
+    cmat <- try(cor(x[, !(colnames(x) %in% c(method$ignore, method$remove)), drop = FALSE],
+                    use = "pairwise.complete.obs"),
                 silent = TRUE)
-    if(class(cmat)[1] != "try-error") {
+    if(!inherits(cmat, "try-error")) {
       high_corr <- findCorrelation(cmat, cutoff = cutoff)
       if(length(high_corr) > 0) {
         removed <- colnames(cmat)[high_corr]
         method$remove <- unique(c(method$remove, removed))
         if(verbose) cat(paste(" ", length(removed), "highly correlated predictors were removed.\n"))
-      } else warning(paste("correlation matrix could not be computed:\n", cmat))
-    }
+      }
+    } else warning(paste("correlation matrix could not be computed:\n", cmat))
     method$corr <- NULL
   }
 
@@ -336,7 +339,7 @@ preProcess.default <- function(x, method = c("center", "scale"),
       if(length(yj) > 0) {
         for(i in seq(along = yj)) {
           who <- names(yj)[i]
-          x[,who] <- recipes::yj_trans(x[,who], yj[who])
+          x[,who] <- recipes::yj_transform(x[,who], yj[who])
         }
       }
     } else {
@@ -461,7 +464,12 @@ preProcess.default <- function(x, method = c("center", "scale"),
     tmp <- prcomp(x[, method$pca, drop = FALSE], scale = TRUE, retx = FALSE)
     if(is.null(pcaComp)) {
       cumVar <- cumsum(tmp$sdev^2/sum(tmp$sdev^2))
-      numComp <- max(2, which.max(cumVar >= thresh))
+      aboveThresh <- cumVar >= thresh
+      if (!any(aboveThresh)) {
+        numComp <- length(aboveThresh)
+      } else {
+        numComp <- max(2, which.max(aboveThresh))
+      }
     } else numComp <- min(pcaComp, ncol(tmp$rotation))
     rot <- tmp$rotation[,1:numComp]
   } else {
@@ -551,7 +559,7 @@ predict.preProcess <- function(object, newdata, ...) {
     if(length(lam) > 0) {
       for(i in seq(along = lam)) {
         who <- names(lam)[i]
-        newdata[,who] <- recipes::yj_trans(newdata[,who], lam[who])
+        newdata[,who] <- recipes::yj_transform(newdata[,who], lam[who])
       }
     }
   }
@@ -601,7 +609,7 @@ predict.preProcess <- function(object, newdata, ...) {
       newdata[!cc, object$method$knnImpute] <- hasMiss
     } else {
       if(is.data.frame(newdata)) {
-        newdata[!cc, object$method$knnImpute] <- as.data.frame(hasMiss)
+        newdata[!cc, object$method$knnImpute] <- as.data.frame(hasMiss, stringsAsFactors = TRUE)
       } else newdata[!cc, object$method$knnImpute] <- as.matrix(hasMiss)
     }
   }
@@ -614,7 +622,7 @@ predict.preProcess <- function(object, newdata, ...) {
                          function(x) any(is.na(x)))
     missingVars <- names(missingVars)[missingVars]
     ## ipred's bagging procedure only allows for data frames
-    if(!is.data.frame(hasMiss)) hasMiss <- as.data.frame(hasMiss)
+    if(!is.data.frame(hasMiss)) hasMiss <- as.data.frame(hasMiss, stringsAsFactors = TRUE)
     for(i in seq(along = missingVars)) {
       preds <- predict(object$bagImp[[missingVars[i]]]$model,
                        hasMiss[, !colnames(hasMiss) %in% missingVars[i], drop = FALSE])
@@ -626,7 +634,7 @@ predict.preProcess <- function(object, newdata, ...) {
       newdata[!cc,] <- hasMiss
     } else {
       if(is.data.frame(newdata)) {
-        newdata[!cc,] <- as.data.frame(hasMiss)
+        newdata[!cc,] <- as.data.frame(hasMiss, stringsAsFactors = TRUE)
       } else newdata[!cc,] <- as.matrix(hasMiss)
     }
   }
@@ -642,7 +650,10 @@ predict.preProcess <- function(object, newdata, ...) {
   if(any(names(object$method) == "pca")) {
     pca_cols <- newdata[, object$method$pca, drop = FALSE]
     pca_cols <-if(is.matrix(pca_cols)) pca_cols %*% object$rotation else as.matrix(pca_cols) %*% object$rotation
-    if(is.data.frame(newdata)) pca_cols <- as.data.frame(pca_cols)
+    if (ncol(pca_cols) == 1) {
+      colnames(pca_cols) <- "PC1"
+    }
+    if(is.data.frame(newdata)) pca_cols <- as.data.frame(pca_cols, stringsAsFactors = TRUE)
     newdata <- cbind(newdata, pca_cols)
     ## normally we get rid of columns that we used to create
     ## the PC's unless we still need them or want them
@@ -662,7 +673,7 @@ predict.preProcess <- function(object, newdata, ...) {
     ##if(object$ica$row.norm) newdata <- apply(newdata, 1, function(u) u/sd(u))
     ica_cols <- ica_cols %*% object$ica$K %*% object$ica$W
     colnames(ica_cols) <- paste("ICA", 1:ncol(object$ica$W), sep = "")
-    if(is.data.frame(newdata)) ica_cols <- as.data.frame(ica_cols)
+    if(is.data.frame(newdata)) ica_cols <- as.data.frame(ica_cols, stringsAsFactors = TRUE)
     newdata <- cbind(newdata, ica_cols)
     ## Same as PCA above
     discard <- object$method$ica
@@ -798,7 +809,7 @@ bagImp <- function(var, x, B = 10) {
   ## (y, X) interface, but the latter would have to
   ## do case-wise deletion of samples from the
   ## training set.
-  if(!is.data.frame(x)) x <- as.data.frame(x)
+  if(!is.data.frame(x)) x <- as.data.frame(x, stringsAsFactors = TRUE)
   mod <- ipred::bagging(as.formula(paste(var, "~.")),
                         data = x,
                         nbagg = B,
