@@ -154,7 +154,7 @@
 #' @references \url{http://topepo.github.io/caret/}
 #'
 #' Kuhn (2008), ``Building Predictive Models in R Using the caret''
-#' (\url{http://www.jstatsoft.org/article/view/v028i05/v28i05.pdf})
+#' (\url{https://www.jstatsoft.org/article/view/v028i05/v28i05.pdf})
 #'
 #' \url{https://topepo.github.io/recipes/}
 #' @keywords models
@@ -302,7 +302,7 @@ train.default <- function(x, y,
                           ...,
                           weights = NULL,
                           metric = ifelse(is.factor(y), "Accuracy", "RMSE"),
-                          maximize = ifelse(metric %in% c("RMSE", "logLoss", "MAE"), FALSE, TRUE),
+                          maximize = ifelse(metric %in% c("RMSE", "logLoss", "MAE", "logLoss"), FALSE, TRUE),
                           trControl = trainControl(),
                           tuneGrid = NULL,
                           tuneLength = ifelse(trControl$method == "none", 1, 3)) {
@@ -317,7 +317,7 @@ train.default <- function(x, y,
   if(is.character(y)) y <- as.factor(y)
 
   if( !is.numeric(y) & !is.factor(y) ){
-    msg <- paste("Please make sure that the outcome column is a factor or numeric .",
+    msg <- paste("Please make sure that the outcome column is a factor or numeric.",
                  "The class(es) of the column:",
                  paste0("'", class(y), "'", collapse = ", "))
 
@@ -374,7 +374,7 @@ train.default <- function(x, y,
     trControl$sampling <- parse_sampling(trControl$sampling)
   }
 
-  if(any(class(x) == "data.table")) x <- as.data.frame(x)
+  if(any(class(x) == "data.table")) x <- as.data.frame(x, stringsAsFactors = TRUE)
   check_dims(x = x, y = y)
   n <- if(class(y)[1] == "Surv") nrow(y) else length(y)
 
@@ -783,7 +783,7 @@ train.default <- function(x, y,
                                        metric = metric,
                                        method = method)
     perfNames <- names(performance)
-    performance <- as.data.frame(t(performance))
+    performance <- as.data.frame(t(performance), stringsAsFactors = TRUE)
     performance <- cbind(performance, tuneGrid)
     performance <- performance[-1,,drop = FALSE]
     tmp <- resampledCM <- NULL
@@ -867,7 +867,10 @@ train.default <- function(x, y,
   if(method == "glmnet") finalModel$lambdaOpt <- bestTune$lambda
 
   if(trControl$returnData) {
-    outData <- if(!is.data.frame(x)) try(as.data.frame(x), silent = TRUE) else x
+    outData <- if (inherits(x, "sparseMatrix")) as.matrix(x) else x
+    if(!is.data.frame(outData)) {
+      outData <- try(as.data.frame(outData, stringsAsFactors = TRUE), silent = TRUE)
+    }
     if(inherits(outData, "try-error")) {
       warning("The training data could not be converted to a data frame for saving")
       outData <- NULL
@@ -920,7 +923,7 @@ train.default <- function(x, y,
 #' @export
 train.formula <- function (form, data, ..., weights, subset, na.action = na.fail, contrasts = NULL)  {
   m <- match.call(expand.dots = FALSE)
-  if (is.matrix(eval.parent(m$data)))  m$data <- as.data.frame(data)
+  if (is.matrix(eval.parent(m$data)))  m$data <- as.data.frame(data, stringsAsFactors = TRUE)
   m$... <- m$contrasts <- NULL
 
   check_na_conflict(match.call(expand.dots = TRUE))
@@ -1370,7 +1373,7 @@ train.recipe <- function(x,
                                        metric = metric,
                                        method = method)
     perfNames <- names(performance)
-    performance <- as.data.frame(t(performance))
+    performance <- as.data.frame(t(performance), stringsAsFactors = TRUE)
     performance <- cbind(performance, tuneGrid)
     performance <- performance[-1,,drop = FALSE]
     tmp <- resampledCM <- NULL
